@@ -32,13 +32,8 @@ class Storage(BaseStorage.BaseStorage,
 
     implements(
         ZODB.interfaces.IStorage,
-        # "restore" missing for the moment, but "store" implements this
-        # interface.
         # ZODB.interfaces.IStorageRestoreable,
-        # XXX: imperfect iterator implementation:
-        # - start & stop are not handled (raises if either is not None)
-        # - transaction isolation is not done
-        # ZODB.interfaces.IStorageIteration,
+        ZODB.interfaces.IStorageIteration,
         ZODB.interfaces.IStorageUndoable,
         ZODB.interfaces.IExternalGC,
         ZODB.interfaces.ReadVerifyingStorage,
@@ -127,6 +122,11 @@ class Storage(BaseStorage.BaseStorage,
             raise POSException.POSKeyError(oid)
 
     def loadBefore(self, oid, tid):
+        # XXX: FileStorage return an empty string for a deleted object
+        #      but it may cause EOFError exceptions in ZODB.Connection
+        #      and it makes impossible to store empty values.
+        #      We think it's wrong behaviour and raise POSKeyError instead.
+        #      Or maybe we should return None?
         try:
             return self.app.load(oid, None, tid)
         except NEOStorageDoesNotExistError:
@@ -134,12 +134,9 @@ class Storage(BaseStorage.BaseStorage,
         except NEOStorageNotFoundError:
             return None
 
-    def iterator(self, start=None, stop=None):
-        # Iterator lives in its own transaction, so get a fresh snapshot.
-        snapshot_tid = self.lastTransaction()
-        if not stop or snapshot_tid < stop:
-            stop = snapshot_tid
-        return self.app.iterator(start, stop)
+    @property
+    def iterator(self):
+        return self.app.iterator
 
     # undo
     def undo(self, transaction_id, txn):
@@ -181,8 +178,7 @@ class Storage(BaseStorage.BaseStorage,
 
     def copyTransactionsFrom(self, source, verbose=False):
         """ Zope compliant API """
-        return self.app.importFrom(source, None, None,
-                self.tryToResolveConflict)
+        return self.importFrom(source)
 
     def importFrom(self, source, start=None, stop=None, preindex=None):
         """ Allow import only a part of the source storage """
