@@ -41,7 +41,6 @@ from .exception import NEOStorageNotFoundError
 from .handlers import storage, master
 from neo.lib.dispatcher import Dispatcher, ForgottenPacket
 from .poll import ThreadedPoll, psThreadedPoll
-from .iterator import Iterator
 from .cache import ClientCache
 from .pool import ConnectionPool
 from neo.lib.util import p64, u64, parseMasterList
@@ -400,11 +399,11 @@ class Application(object):
                 # Do not get something more recent than the last invalidation
                 # we got from master.
                 before_tid = p64(u64(self.last_tid) + 1)
-            result = self._loadFromStorage(oid, tid, before_tid)
+            data, tid, next_tid, _ = self._loadFromStorage(oid, tid, before_tid)
             acquire()
             try:
-                if not (self._loading_oid or result[2]):
-                    result = result[0], result[1], self._loading_invalidated
+                result = data, tid, (next_tid if self._loading_oid or next_tid
+                                              else self._loading_invalidated)
                 self._cache.store(oid, *result)
                 return result
             finally:
@@ -416,7 +415,7 @@ class Application(object):
         packet = Packets.AskObject(oid, at_tid, before_tid)
         for node, conn in self.cp.iterateForObject(oid, readable=True):
             try:
-                noid, tid, next_tid, compression, checksum, data \
+                tid, next_tid, compression, checksum, data, data_tid \
                     = self._askStorage(conn, packet)
             except ConnectionClosed:
                 continue
@@ -426,9 +425,8 @@ class Application(object):
                     logging.error('wrong checksum from %s for oid %s',
                               conn, dump(oid))
                     continue
-                if compression:
-                    data = decompress(data)
-                return data, tid, next_tid
+                return (decompress(data) if compression else data,
+                        tid, next_tid, data_tid)
             raise NEOStorageCreationUndoneError(dump(oid))
         # We didn't got any object from all storage node because of
         # connection error
@@ -797,8 +795,6 @@ class Application(object):
                     data = tryToResolveConflict(oid, current_serial,
                         undone_tid, undo_data, data)
                 except ConflictError:
-                    data = None
-                if data is None:
                     raise UndoError('Some data were modified by a later ' \
                         'transaction', oid)
                 undo_serial = None
@@ -923,28 +919,27 @@ class Application(object):
 
     def importFrom(self, source, start, stop, tryToResolveConflict,
             preindex=None):
+        # TODO: The main difference with BaseStorage implementation is that
+        #       preindex can't be filled with the result 'store' (tid only
+        #       known after 'tpc_finish'. This method could be dropped if we
+        #       implemented IStorageRestoreable (a wrapper around source would
+        #       still be required for partial import).
         if preindex is None:
             preindex = {}
-        transaction_iter = source.iterator(start, stop)
-        for transaction in transaction_iter:
+        for transaction in source.iterator(start, stop):
             tid = transaction.tid
             self.tpc_begin(transaction, tid, transaction.status)
             for r in transaction:
                 oid = r.oid
                 pre = preindex.get(oid)
-                # TODO: bypass conflict resolution, locks...
                 self.store(oid, pre, r.data, r.version, transaction)
                 preindex[oid] = tid
             conflicted = self.tpc_vote(transaction, tryToResolveConflict)
             assert not conflicted, conflicted
             real_tid = self.tpc_finish(transaction, tryToResolveConflict)
             assert real_tid == tid, (real_tid, tid)
-        transaction_iter.close()
 
-    def iterator(self, start, stop):
-        if start is None:
-            start = ZERO_TID
-        return Iterator(self, start, stop)
+    from .iterator import iterator
 
     def lastTransaction(self):
         self._askPrimary(Packets.AskLastTransaction())
