@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2011-2012  Nexedi SA
+# Copyright (C) 2011-2014  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -235,6 +235,22 @@ class ClientCache(object):
             else:
                 assert item.next_tid <= tid, (item, oid, tid)
 
+    def clear_current(self):
+        oid_list = []
+        for oid, item_list in self._oid_dict.items():
+            item = item_list[-1]
+            if item.next_tid is None:
+                self._remove(item)
+                del item_list[-1]
+                # We don't preserve statistics of removed items. This could be
+                # done easily when previous versions are cached, by copying
+                # counters, but it would not be fair for other oids, so it's
+                # probably not worth it.
+                if not item_list:
+                    del self._oid_dict[oid]
+                oid_list.append(oid)
+        return oid_list
+
 
 def test(self):
     cache = ClientCache()
@@ -250,7 +266,11 @@ def test(self):
     data = '15', 15, None
     cache.store(1, *data)
     self.assertEqual(cache.load(1, None), data)
+    self.assertEqual(cache.clear_current(), [1])
+    self.assertEqual(cache.load(1, None), None)
+    cache.store(1, *data)
     cache.invalidate(1, 20)
+    self.assertEqual(cache.clear_current(), [])
     self.assertEqual(cache.load(1, 20), ('15', 15, 20))
     cache.store(1, '10', 10, 15)
     cache.store(1, '20', 20, 21)
