@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2012  Nexedi SA
+# Copyright (C) 2006-2014  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -21,7 +21,6 @@ from neo.lib import logging
 from neo.lib.locking import RLock
 from neo.lib.protocol import NodeTypes, Packets
 from neo.lib.connection import MTClientConnection, ConnectionClosed
-from neo.lib.profiling import profiler_decorator
 from neo.lib.exception import NodeNotReady
 from .exception import NEOStorageError
 
@@ -52,7 +51,6 @@ class ConnectionPool(object):
         self.connection_lock_release = l.release
         self.node_failure_dict = {}
 
-    @profiler_decorator
     def _initNodeConnection(self, node):
         """Init a connection to a given storage node."""
         app = self.app
@@ -75,7 +73,6 @@ class ConnectionPool(object):
             logging.info('Connected %r', node)
         return conn
 
-    @profiler_decorator
     def _dropConnections(self):
         """Drop connections."""
         for conn in self.connection_dict.values():
@@ -93,11 +90,9 @@ class ConnectionPool(object):
             finally:
                 conn.unlock()
 
-    @profiler_decorator
     def notifyFailure(self, node):
         self.node_failure_dict[node.getUUID()] = time.time() + MAX_FAILURE_AGE
 
-    @profiler_decorator
     def getCellSortKey(self, cell):
         uuid = cell.getUUID()
         if uuid in self.connection_dict:
@@ -107,59 +102,68 @@ class ConnectionPool(object):
             return CELL_GOOD
         return CELL_FAILED
 
-    @profiler_decorator
     def getConnForCell(self, cell):
         return self.getConnForNode(cell.getNode())
 
     def iterateForObject(self, object_id, readable=False):
         """ Iterate over nodes managing an object """
-        pt = self.app.getPartitionTable()
+        pt = self.app.pt
         if type(object_id) is str:
             object_id = pt.getPartition(object_id)
         cell_list = pt.getCellList(object_id, readable)
         if not cell_list:
             raise NEOStorageError('no storage available')
         getConnForNode = self.getConnForNode
-        while cell_list:
+        while 1:
             new_cell_list = []
-            cell_list = [c for c in cell_list if c.getNode().isRunning()]
+            # Shuffle to randomise node to access...
             shuffle(cell_list)
+            # ...and sort with non-unique keys, to prioritise ranges of
+            # randomised entries.
             cell_list.sort(key=self.getCellSortKey)
             for cell in cell_list:
                 node = cell.getNode()
-                conn = getConnForNode(node)
-                if conn is not None:
-                    yield (node, conn)
-                elif node.isRunning():
-                    new_cell_list.append(cell)
+                if node.isRunning():
+                    conn = getConnForNode(node)
+                    if conn is not None:
+                        yield (node, conn)
+                    # Re-check if node is running, as our knowledge of its
+                    # state can have changed during connection attempt.
+                    elif node.isRunning():
+                        new_cell_list.append(cell)
+            if not new_cell_list:
+                break
             cell_list = new_cell_list
-            if new_cell_list:
-                # wait a bit to avoid a busy loop
-                time.sleep(1)
+            # wait a bit to avoid a busy loop
+            time.sleep(1)
 
-    @profiler_decorator
     def getConnForNode(self, node):
         """Return a locked connection object to a given node
         If no connection exists, create a new one"""
         if node.isRunning():
             uuid = node.getUUID()
-            self.connection_lock_acquire()
             try:
                 # Already connected to node
                 return self.connection_dict[uuid]
             except KeyError:
-                if len(self.connection_dict) > self.max_pool_size:
-                    # must drop some unused connections
-                    self._dropConnections()
-                # Create new connection to node
-                conn = self._initNodeConnection(node)
-                if conn is not None:
-                    self.connection_dict[uuid] = conn
-                return conn
-            finally:
-                self.connection_lock_release()
+                self.connection_lock_acquire()
+                try:
+                    # Second lookup, if another thread initiated connection
+                    # while we were waiting for connection lock.
+                    try:
+                        return self.connection_dict[uuid]
+                    except KeyError:
+                        if len(self.connection_dict) > self.max_pool_size:
+                            # must drop some unused connections
+                            self._dropConnections()
+                        # Create new connection to node
+                        conn = self._initNodeConnection(node)
+                        if conn is not None:
+                            self.connection_dict[uuid] = conn
+                        return conn
+                finally:
+                    self.connection_lock_release()
 
-    @profiler_decorator
     def removeConnection(self, node):
         """Explicitly remove connection when a node is broken."""
         self.connection_dict.pop(node.getUUID(), None)
