@@ -14,8 +14,27 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from collections import defaultdict
+from functools import wraps
 from neo.lib import logging, util
-from neo.lib.protocol import ZERO_TID
+from neo.lib.protocol import ZERO_TID, BackendNotImplemented
+
+def lazymethod(func):
+    def getter(self):
+        cls = self.__class__
+        name = func.__name__
+        assert name not in cls.__dict__
+        setattr(cls, name, func(self))
+        return getattr(self, name)
+    return property(getter, doc=func.__doc__)
+
+def fallback(func):
+    def warn(self):
+        logging.info("Fallback to generic/slow implementation of %s."
+            " It should be overridden by backend storage (%s).",
+            func.__name__, self.__class__.__name__)
+        return func
+    return lazymethod(wraps(func)(warn))
 
 class CreationUndone(Exception):
     pass
@@ -30,29 +49,43 @@ class DatabaseManager(object):
         self._wait = wait
         self._parse(database)
 
+    def __getattr__(self, attr):
+        if attr == "_getPartition":
+            np = self.getNumPartitions()
+            value = lambda x: x % np
+        else:
+            return self.__getattribute__(attr)
+        setattr(self, attr, value)
+        return value
+
     def _parse(self, database):
         """Called during instanciation, to process database parameter."""
         pass
 
-    def setup(self, reset = 0):
-        """Set up a database
+    def setup(self, reset=0):
+        """Set up a database, discarding existing data first if reset is True
+        """
+        if reset:
+            self.erase()
+        self._uncommitted_data = defaultdict(int)
+        self._setup()
+
+    def _setup(self):
+        """To be overriden by the backend to set up a database
 
         It must recover self._uncommitted_data from temporary object table.
-        _uncommitted_data is a dict containing refcounts to data of
-        write-locked objects, except in case of undo, where the refcount is
-        increased later, when the object is read-locked.
+        _uncommitted_data is already instantiated and must be updated with
+        refcounts to data of write-locked objects, except in case of undo,
+        where the refcount is increased later, when the object is read-locked.
         Keys are data ids and values are number of references.
-
-        If reset is true, existing data must be discarded and
-        self._uncommitted_data must be an empty dict.
         """
         raise NotImplementedError
 
-    def commit(self):
+    def doOperation(self, app):
         pass
 
-    def _getPartition(self, oid_or_tid):
-        return oid_or_tid % self.getNumPartitions()
+    def commit(self):
+        pass
 
     def getConfiguration(self, key):
         """
@@ -97,6 +130,10 @@ class DatabaseManager(object):
             Store the number of partitions into a database.
         """
         self.setConfiguration('partitions', num_partitions)
+        try:
+            del self._getPartition
+        except AttributeError:
+            pass
 
     def getNumReplicas(self):
         """
@@ -149,12 +186,29 @@ class DatabaseManager(object):
         logging.debug('backup_tid = %s', tid)
         return self.setConfiguration('backup_tid', tid)
 
+    def _setPackTID(self, tid):
+        self._setConfiguration('_pack_tid', tid)
+
+    def _getPackTID(self):
+        try:
+            return int(self.getConfiguration('_pack_tid'))
+        except TypeError:
+            return -1
+
     def getPartitionTable(self):
         """Return a whole partition table as a sequence of rows. Each row
         is again a tuple of an offset (row ID), the NID of a storage
         node, and a cell state."""
         raise NotImplementedError
 
+    def getLastTID(self, max_tid):
+        """Return greatest tid in trans table that is <= given 'max_tid'
+
+        Required only to import a DB using Importer backend.
+        max_tid must be in unpacked format.
+        """
+        raise NotImplementedError
+
     def _getLastIDs(self, all=True):
         raise NotImplementedError
 
@@ -179,6 +233,25 @@ class DatabaseManager(object):
         searched from unfinished transactions as well."""
         raise NotImplementedError
 
+    @fallback
+    def getLastObjectTID(self, oid):
+        """Return the latest tid of given oid or None if it does not exist"""
+        r = self.getObject(oid)
+        return r and r[0]
+
+    def _getNextTID(self, partition, oid, tid):
+        """
+        partition (int)
+            Must be the result of (oid % self.getPartition(oid))
+        oid (int)
+            Identifier of object to retrieve.
+        tid (int)
+            Exact serial to retrieve.
+
+        If tid is the last revision of oid, None is returned.
+        """
+        raise NotImplementedError
+
     def _getObject(self, oid, tid=None, before_tid=None):
         """
         oid (int)
@@ -213,39 +286,24 @@ class DatabaseManager(object):
                 - data_serial (packed, None)
         """
         u64 = util.u64
-        p64 = util.p64
-        oid = u64(oid)
-        if tid is not None:
-            tid = u64(tid)
-        if before_tid is not None:
-            before_tid = u64(before_tid)
-        result = self._getObject(oid, tid, before_tid)
-        if result:
-            serial, next_serial, compression, checksum, data, data_serial = \
-                result
-            assert before_tid is None or next_serial is None or \
-                   before_tid <= next_serial
-            if serial is not None:
-                serial = p64(serial)
-            if next_serial is not None:
-                next_serial = p64(next_serial)
-            if data_serial is not None:
-                data_serial = p64(data_serial)
-            return serial, next_serial, compression, checksum, data, data_serial
-        # See if object exists at all
-        return self._getObject(oid) and False
+        r = self._getObject(u64(oid), tid and u64(tid),
+                            before_tid and u64(before_tid))
+        try:
+            serial, next_serial, compression, checksum, data, data_serial = r
+        except TypeError:
+            # See if object exists at all
+            return (tid or before_tid) and self.getLastObjectTID(oid) and False
+        return (util.p64(serial),
+                None if next_serial is None else util.p64(next_serial),
+                compression, checksum, data,
+                None if data_serial is None else util.p64(data_serial))
 
-    def changePartitionTable(self, ptid, cell_list):
+    def changePartitionTable(self, ptid, cell_list, reset=False):
         """Change a part of a partition table. The list of cells is
         a tuple of tuples, each of which consists of an offset (row ID),
         the NID of a storage node, and a cell state. The Partition
-        Table ID must be stored as well."""
-        raise NotImplementedError
-
-    def setPartitionTable(self, ptid, cell_list):
-        """Set a whole partition table. The semantics is the same as
-        changePartitionTable, except that existing data must be
-        thrown away."""
+        Table ID must be stored as well. If reset is True, existing data
+        is first thrown away."""
         raise NotImplementedError
 
     def dropPartitions(self, offset_list):
@@ -276,7 +334,7 @@ class DatabaseManager(object):
         """
         raise NotImplementedError
 
-    def _storeData(self, checksum, data, compression):
+    def storeData(self, checksum, data, compression):
         """To be overriden by the backend to store object raw data
 
         If same data was already stored, the storage only has to check there's
@@ -284,24 +342,24 @@ class DatabaseManager(object):
         """
         raise NotImplementedError
 
-    def storeData(self, checksum_or_id, data=None, compression=None):
-        """Store object raw data
+    def holdData(self, checksum_or_id, *args):
+        """Store raw data of temporary object
 
-        checksum must be the result of neo.lib.util.makeChecksum(data)
-        'compression' indicates if 'data' is compressed.
-        A volatile reference is set to this data until 'unlockData' is called
+        If 'checksum_or_id' is a checksum, it must be the result of
+        makeChecksum(data) and extra parameters must be (data, compression)
+        where 'compression' indicates if 'data' is compressed.
+        A volatile reference is set to this data until 'releaseData' is called
         with this checksum.
         If called with only an id, it only increment the volatile
         reference to the data matching the id.
         """
-        refcount = self._uncommitted_data
-        if data is not None:
-            checksum_or_id = self._storeData(checksum_or_id, data, compression)
-        refcount[checksum_or_id] = 1 + refcount.get(checksum_or_id, 0)
+        if args:
+            checksum_or_id = self.storeData(checksum_or_id, *args)
+        self._uncommitted_data[checksum_or_id] += 1
         return checksum_or_id
 
-    def unlockData(self, data_id_list, prune=False):
-        """Release 1 volatile reference to given list of checksums
+    def releaseData(self, data_id_list, prune=False):
+        """Release 1 volatile reference to given list of data ids
 
         If 'prune' is true, any data that is not referenced anymore (either by
         a volatile reference or by a fully-committed object) is deleted.
@@ -317,35 +375,23 @@ class DatabaseManager(object):
             self._pruneData(data_id_list)
             self.commit()
 
-    __getDataTID = set()
+    @fallback
     def _getDataTID(self, oid, tid=None, before_tid=None):
         """
         Return a 2-tuple:
         tid (int)
             tid corresponding to received parameters
         serial
-            tid at which actual object data is located
+            data tid of the found record
 
-        If 'tid is None', requested object and transaction could
-        not be found.
-        If 'serial is None', requested object exist but has no data (its creation
-        has been undone).
-        If 'tid == serial', it means that requested transaction
-        contains object data.
-        Otherwise, it's an undo transaction which did not involve conflict
-        resolution.
+        (None, None) is returned if requested object and transaction
+        could not be found.
+
+        This method only exists for performance reasons, by not returning data:
+        _getObject already returns these values but it is slower.
         """
-        if self.__class__ not in self.__getDataTID:
-            self.__getDataTID.add(self.__class__)
-            logging.warning("Fallback to generic/slow implementation"
-                " of _getDataTID. It should be overriden by backend storage.")
         r = self._getObject(oid, tid, before_tid)
-        if r:
-            serial, _, _, data_id, _, value_serial = r
-            if value_serial is None and data_id:
-                return serial, serial
-            return serial, value_serial
-        return None, None
+        return (r[0], r[-1]) if r else (None, None)
 
     def findUndoTID(self, oid, tid, ltid, undone_tid, transaction_object):
         """
@@ -385,17 +431,17 @@ class DatabaseManager(object):
             ltid = u64(ltid)
         undone_tid = u64(undone_tid)
         def getDataTID(tid=None, before_tid=None):
-            tid, value_serial = self._getDataTID(oid, tid, before_tid)
-            if value_serial not in (None, tid):
-                if value_serial >= tid:
-                    raise ValueError("Incorrect value reference found for"
-                                     " oid %d at tid %d: reference = %d"
-                                     % (oid, value_serial, tid))
-                if value_serial != getDataTID(value_serial)[1]:
-                    logging.warning("Multiple levels of indirection"
-                        " when getting data serial for oid %d at tid %d."
-                        " This causes suboptimal performance.", oid, tid)
-            return tid, value_serial
+            tid, data_tid = self._getDataTID(oid, tid, before_tid)
+            current_tid = tid
+            while data_tid:
+                if data_tid < tid:
+                    tid, data_tid = self._getDataTID(oid, data_tid)
+                    if tid is not None:
+                        continue
+                logging.error("Incorrect data serial for oid %s at tid %s",
+                              oid, current_tid)
+                return current_tid, current_tid
+            return current_tid, tid
         if transaction_object:
             current_tid = current_data_tid = u64(transaction_object[2])
         else:
@@ -449,12 +495,12 @@ class DatabaseManager(object):
         area as well."""
         raise NotImplementedError
 
-    def getObjectHistory(self, oid, offset = 0, length = 1):
+    def getObjectHistory(self, oid, offset, length):
         """Return a list of serials and sizes for a given object ID.
         The length specifies the maximum size of such a list. Result starts
         with latest serial, and the list must be sorted in descending order.
         If there is no such object ID in a database, return None."""
-        raise NotImplementedError
+        raise BackendNotImplemented(self.getObjectHistory)
 
     def getReplicationObjectList(self, min_tid, max_tid, length, partition,
             min_oid):
@@ -467,7 +513,7 @@ class DatabaseManager(object):
         """Return a list of TIDs in ascending order from an offset,
         at most the specified length. The list of partitions are passed
         to filter out non-applicable TIDs."""
-        raise NotImplementedError
+        raise BackendNotImplemented(self.getTIDList)
 
     def getReplicationTIDList(self, min_tid, max_tid, length, partition):
         """Return a list of TIDs in ascending order from an initial tid value,
