@@ -16,9 +16,9 @@
 
 from binascii import a2b_hex
 import MySQLdb
-from MySQLdb import IntegrityError, OperationalError
+from MySQLdb import DataError, IntegrityError, OperationalError
 from MySQLdb.constants.CR import SERVER_GONE_ERROR, SERVER_LOST
-from MySQLdb.constants.ER import DUP_ENTRY
+from MySQLdb.constants.ER import DATA_TOO_LONG, DUP_ENTRY
 from array import array
 from hashlib import sha1
 import re
@@ -44,7 +44,7 @@ def splitOIDField(tid, oids):
 class MySQLDatabaseManager(DatabaseManager):
     """This class manages a database on MySQL."""
 
-    # WARNING: some parts are not concurrent safe (ex: storeData)
+    # WARNING: some parts are not concurrent safe (ex: holdData)
     # (there must be only 1 writable connection per DB)
 
     # Disabled even on MySQL 5.1-5.5 and MariaDB 5.2-5.3 because
@@ -82,13 +82,13 @@ class MySQLDatabaseManager(DatabaseManager):
         while True:
             try:
                 self.conn = MySQLdb.connect(**kwd)
+                break
             except Exception:
                 if timeout_at is not None and time.time() >= timeout_at:
                     raise
                 logging.exception('Connection to MySQL failed, retrying.')
                 time.sleep(1)
-            else:
-                break
+        self._active = 0
         self.conn.autocommit(False)
         self.conn.query("SET SESSION group_concat_max_len = %u" % (2**32-1))
         self.conn.set_sql_mode("TRADITIONAL,NO_ENGINE_SUBSTITUTION")
@@ -96,6 +96,7 @@ class MySQLDatabaseManager(DatabaseManager):
     def commit(self):
         logging.debug('committing...')
         self.conn.commit()
+        self._active = 0
 
     def query(self, query):
         """Query data from a database."""
@@ -111,37 +112,36 @@ class MySQLDatabaseManager(DatabaseManager):
                 logging.debug('querying %s...', query_part)
 
             conn.query(query)
-            r = conn.store_result()
-            if r is not None:
-                new_r = []
-                for row in r.fetch_row(r.num_rows()):
-                    new_row = []
-                    for d in row:
-                        if isinstance(d, array):
-                            d = d.tostring()
-                        new_row.append(d)
-                    new_r.append(tuple(new_row))
-                r = tuple(new_r)
-
+            if query.startswith("SELECT "):
+                r = conn.store_result()
+                return tuple([
+                    tuple([d.tostring() if isinstance(d, array) else d
+                           for d in row])
+                    for row in r.fetch_row(r.num_rows())])
+            r = query.split(None, 1)[0]
+            if r in ("INSERT", "REPLACE", "DELETE", "UPDATE"):
+                self._active = 1
+            else:
+                assert r in ("ALTER", "CREATE", "DROP", "TRUNCATE"), query
         except OperationalError, m:
-            if m[0] in (SERVER_GONE_ERROR, SERVER_LOST):
+            if m[0] in (SERVER_GONE_ERROR, SERVER_LOST) and not self._active:
                 logging.info('the MySQL server is gone; reconnecting')
                 self._connect()
                 return self.query(query)
             raise DatabaseFailure('MySQL error %d: %s' % (m[0], m[1]))
-        return r
 
-    def escape(self, s):
+    @property
+    def escape(self):
         """Escape special characters in a string."""
-        return self.conn.escape_string(s)
+        return self.conn.escape_string
 
-    def setup(self, reset = 0):
+    def erase(self):
+        self.query(
+            "DROP TABLE IF EXISTS config, pt, trans, obj, data, ttrans, tobj")
+
+    def _setup(self):
         self._config.clear()
         q = self.query
-
-        if reset:
-            q('DROP TABLE IF EXISTS config, pt, trans, obj, data, ttrans, tobj')
-
         # The table "config" stores configuration parameters which affect the
         # persistent data.
         q("""CREATE TABLE IF NOT EXISTS config (
@@ -217,8 +217,8 @@ class MySQLDatabaseManager(DatabaseManager):
                  PRIMARY KEY (tid, oid)
              ) ENGINE = InnoDB""")
 
-        self._uncommitted_data = dict(q("SELECT data_id, count(*)"
-            " FROM tobj WHERE data_id IS NOT NULL GROUP BY data_id") or ())
+        self._uncommitted_data.update(q("SELECT data_id, count(*)"
+            " FROM tobj WHERE data_id IS NOT NULL GROUP BY data_id"))
 
     def getConfiguration(self, key):
         try:
@@ -237,34 +237,36 @@ class MySQLDatabaseManager(DatabaseManager):
         q = self.query
         e = self.escape
         self._config[key] = value
-        key = e(str(key))
+        k = e(str(key))
         if value is None:
-            q("DELETE FROM config WHERE name = '%s'" % key)
-        else:
-            value = e(str(value))
-            q("REPLACE INTO config VALUES ('%s', '%s')" % (key, value))
-
-    def _setPackTID(self, tid):
-        self._setConfiguration('_pack_tid', tid)
-
-    def _getPackTID(self):
+            q("DELETE FROM config WHERE name = '%s'" % k)
+            return
+        value = str(value)
+        sql = "REPLACE INTO config VALUES ('%s', '%s')" % (k, e(value))
         try:
-            return int(self.getConfiguration('_pack_tid'))
-        except TypeError:
-            return -1
+            q(sql)
+        except DataError, (code, _):
+            if code != DATA_TOO_LONG or len(value) < 256 or key != "zodb":
+                raise
+            q("ALTER TABLE config MODIFY value VARBINARY(%s) NULL" % len(value))
+            q(sql)
 
     def getPartitionTable(self):
         return self.query("SELECT * FROM pt")
 
+    def getLastTID(self, max_tid):
+        return self.query("SELECT MAX(tid) FROM trans WHERE tid<=%s"
+                          % max_tid)[0][0]
+
     def _getLastIDs(self, all=True):
         p64 = util.p64
         q = self.query
-        trans = dict((partition, p64(tid))
+        trans = {partition: p64(tid)
             for partition, tid in q("SELECT partition, MAX(tid)"
-                                    " FROM trans GROUP BY partition"))
-        obj = dict((partition, p64(tid))
+                                    " FROM trans GROUP BY partition")}
+        obj = {partition: p64(tid)
             for partition, tid in q("SELECT partition, MAX(tid)"
-                                    " FROM obj GROUP BY partition"))
+                                    " FROM obj GROUP BY partition")}
         oid = q("SELECT MAX(oid) FROM (SELECT MAX(oid) AS oid FROM obj"
                                       " GROUP BY partition) as t")[0][0]
         if all:
@@ -291,6 +293,20 @@ class MySQLDatabaseManager(DatabaseManager):
                  % (self._getPartition(oid), oid, tid)) or all and \
                q("SELECT 1 FROM tobj WHERE tid=%d AND oid=%d" % (tid, oid))
 
+    def getLastObjectTID(self, oid):
+        oid = util.u64(oid)
+        r = self.query("SELECT tid FROM obj"
+                       " WHERE partition=%d AND oid=%d"
+                       " ORDER BY tid DESC LIMIT 1"
+                       % (self._getPartition(oid), oid))
+        return util.p64(r[0][0]) if r else None
+
+    def _getNextTID(self, *args): # partition, oid, tid
+        r = self.query("SELECT tid FROM obj"
+                       " WHERE partition=%d AND oid=%d AND tid>%d"
+                       " ORDER BY tid LIMIT 1" % args)
+        return r[0][0] if r else None
+
     def _getObject(self, oid, tid=None, before_tid=None):
         q = self.query
         partition = self._getPartition(oid)
@@ -310,12 +326,10 @@ class MySQLDatabaseManager(DatabaseManager):
             serial, compression, checksum, data, value_serial = r[0]
         except IndexError:
             return None
-        r = q("SELECT tid FROM obj WHERE partition=%d AND oid=%d AND tid>%d"
-              " ORDER BY tid LIMIT 1" % (partition, oid, serial))
-        return (serial, r[0][0] if r else None, compression, checksum, data,
-            value_serial)
+        return (serial, self._getNextTID(partition, oid, serial),
+                compression, checksum, data, value_serial)
 
-    def doSetPartitionTable(self, ptid, cell_list, reset):
+    def changePartitionTable(self, ptid, cell_list, reset=False):
         offset_list = []
         q = self.query
         if reset:
@@ -343,12 +357,6 @@ class MySQLDatabaseManager(DatabaseManager):
                         if code != 1517: # duplicate partition name
                             raise
 
-    def changePartitionTable(self, ptid, cell_list):
-        self.doSetPartitionTable(ptid, cell_list, False)
-
-    def setPartitionTable(self, ptid, cell_list):
-        self.doSetPartitionTable(ptid, cell_list, True)
-
     def dropPartitions(self, offset_list):
         q = self.query
         # XXX: these queries are inefficient (execution time increase with
@@ -377,7 +385,7 @@ class MySQLDatabaseManager(DatabaseManager):
         data_id_list = [x for x, in q("SELECT data_id FROM tobj") if x]
         q("TRUNCATE tobj")
         q("TRUNCATE ttrans")
-        self.unlockData(data_id_list, True)
+        self.releaseData(data_id_list, True)
 
     def storeTransaction(self, tid, object_list, transaction, temporary = True):
         e = self.escape
@@ -399,7 +407,7 @@ class MySQLDatabaseManager(DatabaseManager):
                     " WHERE partition=%d AND oid=%d AND tid=%d"
                     % (partition, oid, value_serial))
                 if temporary:
-                    self.storeData(data_id)
+                    self.holdData(data_id)
             else:
                 value_serial = 'NULL'
             q("REPLACE INTO %s VALUES (%d, %d, %d, %s, %s)" % (obj_table,
@@ -422,7 +430,7 @@ class MySQLDatabaseManager(DatabaseManager):
                 " WHERE id IN (%s) AND data_id IS NULL"
                 % ",".join(map(str, data_id_list)))
 
-    def _storeData(self, checksum, data, compression):
+    def storeData(self, checksum, data, compression):
         e = self.escape
         checksum = e(checksum)
         try:
@@ -438,7 +446,7 @@ class MySQLDatabaseManager(DatabaseManager):
         return self.conn.insert_id()
 
     def _getDataTID(self, oid, tid=None, before_tid=None):
-        sql = ('SELECT tid, data_id, value_tid FROM obj'
+        sql = ('SELECT tid, value_tid FROM obj'
                ' WHERE partition = %d AND oid = %d'
               ) % (self._getPartition(oid), oid)
         if tid is not None:
@@ -450,12 +458,7 @@ class MySQLDatabaseManager(DatabaseManager):
             # MySQL does not use an index for a HAVING clause!
             sql += ' ORDER BY tid DESC LIMIT 1'
         r = self.query(sql)
-        if r:
-            (serial, data_id, value_serial), = r
-            if value_serial is None and data_id:
-                return serial, serial
-            return serial, value_serial
-        return None, None
+        return r[0] if r else (None, None)
 
     def finishTransaction(self, tid):
         q = self.query
@@ -466,7 +469,7 @@ class MySQLDatabaseManager(DatabaseManager):
         q("DELETE FROM tobj WHERE tid=%d" % tid)
         q("INSERT INTO trans SELECT * FROM ttrans WHERE tid=%d" % tid)
         q("DELETE FROM ttrans WHERE tid=%d" % tid)
-        self.unlockData(data_id_list)
+        self.releaseData(data_id_list)
         self.commit()
 
     def deleteTransaction(self, tid, oid_list=()):
@@ -476,7 +479,7 @@ class MySQLDatabaseManager(DatabaseManager):
         q = self.query
         sql = " FROM tobj WHERE tid=%d" % tid
         data_id_list = [x for x, in q("SELECT data_id" + sql) if x]
-        self.unlockData(data_id_list)
+        self.releaseData(data_id_list)
         q("DELETE" + sql)
         q("""DELETE FROM ttrans WHERE tid = %d""" % tid)
         q("""DELETE FROM trans WHERE partition = %d AND tid = %d""" %
@@ -531,45 +534,19 @@ class MySQLDatabaseManager(DatabaseManager):
             oid_list = splitOIDField(tid, oids)
             return oid_list, user, desc, ext, bool(packed), util.p64(ttid)
 
-    def _getObjectLength(self, oid, value_serial):
-        if value_serial is None:
-            raise CreationUndone
-        r = self.query("""SELECT LENGTH(value), value_tid
-                    FROM obj LEFT JOIN data ON (obj.data_id = data.id)
-                    WHERE partition = %d AND oid = %d AND tid = %d""" %
-            (self._getPartition(oid), oid, value_serial))
-        length, value_serial = r[0]
-        if length is None:
-            logging.info("Multiple levels of indirection when "
-                "searching for object data for oid %d at tid %d."
-                " This causes suboptimal performance.", oid, value_serial)
-            length = self._getObjectLength(oid, value_serial)
-        return length
-
-    def getObjectHistory(self, oid, offset = 0, length = 1):
+    def getObjectHistory(self, oid, offset, length):
         # FIXME: This method doesn't take client's current ransaction id as
         # parameter, which means it can return transactions in the future of
         # client's transaction.
         oid = util.u64(oid)
         p64 = util.p64
-        pack_tid = self._getPackTID()
-        r = self.query("""SELECT tid, LENGTH(value), value_tid
+        r = self.query("""SELECT tid, LENGTH(value)
                     FROM obj LEFT JOIN data ON (obj.data_id = data.id)
                     WHERE partition = %d AND oid = %d AND tid >= %d
-                    ORDER BY tid DESC LIMIT %d, %d""" \
-                % (self._getPartition(oid), oid, pack_tid, offset, length))
+                    ORDER BY tid DESC LIMIT %d, %d""" %
+            (self._getPartition(oid), oid, self._getPackTID(), offset, length))
         if r:
-            result = []
-            append = result.append
-            for serial, length, value_serial in r:
-                if length is None:
-                    try:
-                        length = self._getObjectLength(oid, value_serial)
-                    except CreationUndone:
-                        length = 0
-                append((p64(serial), length))
-            return result
-        return None
+            return [(p64(tid), length or 0) for tid, length in r]
 
     def getReplicationObjectList(self, min_tid, max_tid, length, partition,
             min_oid):
