@@ -25,6 +25,15 @@ from neo.lib import logging, util
 from neo.lib.exception import DatabaseFailure
 from neo.lib.protocol import CellStates, ZERO_OID, ZERO_TID, ZERO_HASH
 
+def unique_constraint_message(table, column):
+    c = sqlite3.connect(":memory:")
+    c.execute("CREATE TABLE %s (%s UNIQUE)" % (table, column))
+    try:
+        c.executemany("INSERT INTO %s VALUES(?)" % table, 'xx')
+    except sqlite3.IntegrityError, e:
+        return e.args[0]
+    assert False
+
 def splitOIDField(tid, oids):
     if (len(oids) % 8) != 0 or len(oids) == 0:
         raise DatabaseFailure('invalid oids length for tid %d: %d' % (tid,
@@ -92,14 +101,13 @@ class SQLiteDatabaseManager(DatabaseManager):
     else:
         query = property(lambda self: self.conn.execute)
 
-    def setup(self, reset = 0):
+    def erase(self):
+        for t in 'config', 'pt', 'trans', 'obj', 'data', 'ttrans', 'tobj':
+            self.query('DROP TABLE IF EXISTS ' + t)
+
+    def _setup(self):
         self._config.clear()
         q = self.query
-
-        if reset:
-            for t in 'config', 'pt', 'trans', 'obj', 'data', 'ttrans', 'tobj':
-                q('DROP TABLE IF EXISTS ' + t)
-
         # The table "config" stores configuration parameters which affect the
         # persistent data.
         q("""CREATE TABLE IF NOT EXISTS config (
@@ -174,7 +182,7 @@ class SQLiteDatabaseManager(DatabaseManager):
                  PRIMARY KEY (tid, oid))
           """)
 
-        self._uncommitted_data = dict(q("SELECT data_id, count(*)"
+        self._uncommitted_data.update(q("SELECT data_id, count(*)"
             " FROM tobj WHERE data_id IS NOT NULL GROUP BY data_id"))
 
     def getConfiguration(self, key):
@@ -197,27 +205,22 @@ class SQLiteDatabaseManager(DatabaseManager):
         else:
             q("REPLACE INTO config VALUES (?,?)", (key, str(value)))
 
-    def _setPackTID(self, tid):
-        self._setConfiguration('_pack_tid', tid)
-
-    def _getPackTID(self):
-        try:
-            return int(self.getConfiguration('_pack_tid'))
-        except TypeError:
-            return -1
-
     def getPartitionTable(self):
         return self.query("SELECT * FROM pt")
 
+    def getLastTID(self, max_tid):
+        return self.query("SELECT MAX(tid) FROM trans WHERE tid<=?",
+                          (max_tid,)).next()[0]
+
     def _getLastIDs(self, all=True):
         p64 = util.p64
         q = self.query
-        trans = dict((partition, p64(tid))
+        trans = {partition: p64(tid)
             for partition, tid in q("SELECT partition, MAX(tid)"
-                                    " FROM trans GROUP BY partition"))
-        obj = dict((partition, p64(tid))
+                                    " FROM trans GROUP BY partition")}
+        obj = {partition: p64(tid)
             for partition, tid in q("SELECT partition, MAX(tid)"
-                                    " FROM obj GROUP BY partition"))
+                                    " FROM obj GROUP BY partition")}
         oid = q("SELECT MAX(oid) FROM (SELECT MAX(oid) AS oid FROM obj"
                                       " GROUP BY partition) as t").next()[0]
         if all:
@@ -245,6 +248,20 @@ class SQLiteDatabaseManager(DatabaseManager):
                q("SELECT 1 FROM tobj WHERE tid=? AND oid=?",
                  (tid, oid)).fetchone()
 
+    def getLastObjectTID(self, oid):
+        oid = util.u64(oid)
+        r = self.query("SELECT tid FROM obj"
+                       " WHERE partition=? AND oid=?"
+                       " ORDER BY tid DESC LIMIT 1",
+                       (self._getPartition(oid), oid)).fetchone()
+        return r and util.p64(r[0])
+
+    def _getNextTID(self, *args): # partition, oid, tid
+        r = self.query("""SELECT tid FROM obj
+                          WHERE partition=? AND oid=? AND tid>?
+                          ORDER BY tid LIMIT 1""", args).fetchone()
+        return r and r[0]
+
     def _getObject(self, oid, tid=None, before_tid=None):
         q = self.query
         partition = self._getPartition(oid)
@@ -262,16 +279,13 @@ class SQLiteDatabaseManager(DatabaseManager):
             serial, compression, checksum, data, value_serial = r.fetchone()
         except TypeError:
             return None
-        r = q("""SELECT tid FROM obj
-                 WHERE partition=? AND oid=? AND tid>?
-                 ORDER BY tid LIMIT 1""",
-              (partition, oid, serial)).fetchone()
         if checksum:
             checksum = str(checksum)
             data = str(data)
-        return serial, r and r[0], compression, checksum, data, value_serial
+        return (serial, self._getNextTID(partition, oid, serial),
+                compression, checksum, data, value_serial)
 
-    def doSetPartitionTable(self, ptid, cell_list, reset):
+    def changePartitionTable(self, ptid, cell_list, reset=False):
         q = self.query
         if reset:
             q("DELETE FROM pt")
@@ -288,12 +302,6 @@ class SQLiteDatabaseManager(DatabaseManager):
                   (offset, nid, int(state)))
         self.setPTID(ptid)
 
-    def changePartitionTable(self, ptid, cell_list):
-        self.doSetPartitionTable(ptid, cell_list, False)
-
-    def setPartitionTable(self, ptid, cell_list):
-        self.doSetPartitionTable(ptid, cell_list, True)
-
     def dropPartitions(self, offset_list):
         where = " WHERE partition=?"
         q = self.query
@@ -310,7 +318,7 @@ class SQLiteDatabaseManager(DatabaseManager):
         data_id_list = [x for x, in q("SELECT data_id FROM tobj") if x]
         q("DELETE FROM tobj")
         q("DELETE FROM ttrans")
-        self.unlockData(data_id_list, True)
+        self.releaseData(data_id_list, True)
 
     def storeTransaction(self, tid, object_list, transaction, temporary=True):
         u64 = util.u64
@@ -327,7 +335,7 @@ class SQLiteDatabaseManager(DatabaseManager):
                     " WHERE partition=? AND oid=? AND tid=?",
                     (partition, oid, value_serial))
                 if temporary:
-                    self.storeData(data_id)
+                    self.holdData(data_id)
             try:
                 q(obj_sql, (partition, oid, tid, data_id, value_serial))
             except sqlite3.IntegrityError:
@@ -360,13 +368,14 @@ class SQLiteDatabaseManager(DatabaseManager):
             q("DELETE FROM data WHERE id IN (%s)"
               % ",".join(map(str, data_id_list)))
 
-    def _storeData(self, checksum, data, compression):
+    def storeData(self, checksum, data, compression,
+                   _dup_hash=unique_constraint_message("data", "hash")):
         H = buffer(checksum)
         try:
             return self.query("INSERT INTO data VALUES (NULL,?,?,?)",
                 (H, compression,  buffer(data))).lastrowid
         except sqlite3.IntegrityError, e:
-            if e.args[0] == 'column hash is not unique':
+            if e.args[0] == _dup_hash:
                 (r, c, d), = self.query("SELECT id, compression, value"
                               " FROM data WHERE hash=?",  (H,))
                 if c == compression and str(d) == data:
@@ -375,7 +384,7 @@ class SQLiteDatabaseManager(DatabaseManager):
 
     def _getDataTID(self, oid, tid=None, before_tid=None):
         partition = self._getPartition(oid)
-        sql = 'SELECT tid, data_id, value_tid FROM obj' \
+        sql = 'SELECT tid, value_tid FROM obj' \
               ' WHERE partition=? AND oid=?'
         if tid is not None:
             r = self.query(sql + ' AND tid=?', (partition, oid, tid))
@@ -386,12 +395,7 @@ class SQLiteDatabaseManager(DatabaseManager):
             r = self.query(sql + ' ORDER BY tid DESC LIMIT 1',
                            (partition, oid))
         r = r.fetchone()
-        if r:
-            serial, data_id, value_serial = r
-            if value_serial is None and data_id:
-                return serial, serial
-            return serial, value_serial
-        return None, None
+        return r or (None, None)
 
     def finishTransaction(self, tid):
         args = util.u64(tid),
@@ -402,7 +406,7 @@ class SQLiteDatabaseManager(DatabaseManager):
         q("DELETE FROM tobj WHERE tid=?", args)
         q("INSERT OR FAIL INTO trans SELECT * FROM ttrans WHERE tid=?", args)
         q("DELETE FROM ttrans WHERE tid=?", args)
-        self.unlockData(data_id_list)
+        self.releaseData(data_id_list)
         self.commit()
 
     def deleteTransaction(self, tid, oid_list=()):
@@ -412,7 +416,7 @@ class SQLiteDatabaseManager(DatabaseManager):
         q = self.query
         sql = " FROM tobj WHERE tid=?"
         data_id_list = [x for x, in q("SELECT data_id" + sql, (tid,)) if x]
-        self.unlockData(data_id_list)
+        self.releaseData(data_id_list)
         q("DELETE" + sql, (tid,))
         q("DELETE FROM ttrans WHERE tid=?", (tid,))
         q("DELETE FROM trans WHERE partition=? AND tid=?",
@@ -472,42 +476,19 @@ class SQLiteDatabaseManager(DatabaseManager):
             return splitOIDField(tid, oids), str(user), \
                 str(description), str(ext), packed, util.p64(ttid)
 
-    def _getObjectLength(self, oid, value_serial):
-        if value_serial is None:
-            raise CreationUndone
-        length, value_serial = self.query("""SELECT LENGTH(value), value_tid
-            FROM obj LEFT JOIN data ON obj.data_id=data.id
-            WHERE partition=? AND oid=? AND tid=?""",
-            (self._getPartition(oid), oid, value_serial)).fetchone()
-        if length is None:
-            logging.info("Multiple levels of indirection"
-                " when searching for object data for oid %d at tid %d."
-                " This causes suboptimal performance.", oid, value_serial)
-            length = self._getObjectLength(oid, value_serial)
-        return length
-
-    def getObjectHistory(self, oid, offset=0, length=1):
+    def getObjectHistory(self, oid, offset, length):
         # FIXME: This method doesn't take client's current transaction id as
         # parameter, which means it can return transactions in the future of
         # client's transaction.
         p64 = util.p64
         oid = util.u64(oid)
-        pack_tid = self._getPackTID()
-        result = []
-        append = result.append
-        for serial, length, value_serial in self.query("""\
-                SELECT tid, LENGTH(value), value_tid
+        return [(p64(tid), length or 0) for tid, length in self.query("""\
+            SELECT tid, LENGTH(value)
                 FROM obj LEFT JOIN data ON obj.data_id = data.id
                 WHERE partition=? AND oid=? AND tid>=?
                 ORDER BY tid DESC LIMIT ?,?""",
-                (self._getPartition(oid), oid, pack_tid, offset, length)):
-            if length is None:
-                try:
-                    length = self._getObjectLength(oid, value_serial)
-                except CreationUndone:
-                    length = 0
-            append((p64(serial), length))
-        return result or None
+            (self._getPartition(oid), oid, self._getPackTID(), offset, length))
+            ] or None
 
     def getReplicationObjectList(self, min_tid, max_tid, length, partition,
             min_oid):
