@@ -16,6 +16,7 @@
 
 import __builtin__
 import errno
+import functools
 import os
 import random
 import socket
@@ -31,11 +32,28 @@ from neo.lib.protocol import NodeTypes, Packets, UUID_NAMESPACES
 from neo.lib.util import getAddressType
 from time import time
 from struct import pack, unpack
+from unittest.case import _ExpectedFailure, _UnexpectedSuccess
 try:
     from ZODB.utils import newTid
 except ImportError:
     pass
 
+def expectedFailure(exception=AssertionError):
+    def decorator(func):
+        def wrapper(*args, **kw):
+            try:
+                func(*args, **kw)
+            except exception, e:
+                # XXX: passing sys.exc_info() causes deadlocks
+                raise _ExpectedFailure((type(e), None, None))
+            raise _UnexpectedSuccess
+        return functools.wraps(func)(wrapper)
+    if callable(exception) and not isinstance(exception, type):
+        func = exception
+        exception = Exception
+        return decorator(func)
+    return decorator
+
 DB_PREFIX = os.getenv('NEO_DB_PREFIX', 'test_neo')
 DB_ADMIN = os.getenv('NEO_DB_ADMIN', 'root')
 DB_PASSWD = os.getenv('NEO_DB_PASSWD', '')
@@ -117,8 +135,6 @@ def setupMySQLdb(db_list, user=DB_USER, password='', clear_databases=True):
 class NeoTestBase(unittest.TestCase):
 
     def setUp(self):
-        sys.stdout.write(' * %s ' % (self.id(), ))
-        sys.stdout.flush()
         logging.name = self.setupLog()
         unittest.TestCase.setUp(self)
 
@@ -126,17 +142,15 @@ class NeoTestBase(unittest.TestCase):
         test_case, logging.name = self.id().rsplit('.', 1)
         logging.setup(os.path.join(getTempDirectory(), test_case + '.log'))
 
-    def tearDown(self,
-            success='ok' if sys.version_info < (2, 7) else 'success'):
+    def tearDown(self):
         assert self.tearDown.im_func is NeoTestBase.tearDown.im_func
-        self._tearDown(sys._getframe(1).f_locals[success])
+        self._tearDown(sys._getframe(1).f_locals['success'])
 
     def _tearDown(self, success):
         # Kill all unfinished transactions for next test.
         # Note we don't even abort them because it may require a valid
         # connection to a master node (see Storage.sync()).
         transaction.manager.__init__()
-        print
 
     class failureException(AssertionError):
         def __init__(self, msg=None):
@@ -514,28 +528,3 @@ class DoNothingConnector(Mock):
 
 __builtin__.pdb = lambda depth=0: \
     debug.getPdb().set_trace(sys._getframe(depth+1))
-
-def _fixMockForInspect():
-    """
-    inspect module change broke Mock, see http://bugs.python.org/issue1785
-    Monkey-patch Mock class if needed by replacing predicate parameter on 2nd
-    getmembers call with isroutine (was ismethod).
-    """
-    import inspect
-    class A(object):
-        def f(self):
-            pass
-    if not inspect.getmembers(A, inspect.ismethod):
-        from mock import MockCallable
-        # _setupSubclassMethodInterceptors is under the FreeBSD license.
-        # See pyMock module for the whole license.
-        def _setupSubclassMethodInterceptors(self):
-            methods = inspect.getmembers(self.__class__, inspect.isroutine)
-            baseMethods = dict(inspect.getmembers(Mock, inspect.isroutine))
-            for m in methods:
-                name = m[0]
-                # Don't record calls to methods of Mock base class.
-                if not name in baseMethods:
-                    self.__dict__[name] = MockCallable(name, self, handcrafted=True)
-        Mock._setupSubclassMethodInterceptors = _setupSubclassMethodInterceptors
-_fixMockForInspect()
