@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2009-2012  Nexedi SA
+# Copyright (C) 2009-2014  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -29,7 +29,8 @@ class MasterHandlerTests(NeoUnitTestBase):
     def setUp(self):
         super(MasterHandlerTests, self).setUp()
         self.db = Mock()
-        self.app = Mock({'getDB': self.db})
+        self.app = Mock({'getDB': self.db,
+                         'txn_contexts': ()})
         self.app.nm = NodeManager()
         self.app.dispatcher = Mock()
         self._next_port = 3000
@@ -167,7 +168,7 @@ class MasterAnswersHandlerTests(MasterHandlerTests):
         conn = self.getFakeConnection()
         oid1, oid2, oid3 = self.getOID(0), self.getOID(1), self.getOID(2)
         self.handler.answerNewOIDs(conn, [oid1, oid2, oid3])
-        self.assertEqual(self.app.new_oid_list, [oid1, oid2, oid3])
+        self.assertEqual(self.app.new_oid_list, [oid3, oid2, oid1])
 
     def test_answerTransactionFinished(self):
         conn = self.getFakeConnection()
