@@ -19,6 +19,7 @@
 import os, random, socket, sys, tempfile, threading, time, types, weakref
 import traceback
 from collections import deque
+from ConfigParser import SafeConfigParser
 from contextlib import contextmanager
 from itertools import count
 from functools import wraps
@@ -166,9 +167,9 @@ class Node(object):
 
     def getConnectionList(self, *peers):
         addr = lambda c: c and (c.accepted_from or c.getAddress())
-        addr_set = set(addr(c.connector) for peer in peers
+        addr_set = {addr(c.connector) for peer in peers
             for c in peer.em.connection_dict.itervalues()
-            if isinstance(c, Connection))
+            if isinstance(c, Connection)}
         addr_set.discard(None)
         return (c for c in self.em.connection_dict.itervalues()
             if isinstance(c, Connection) and addr(c.connector) in addr_set)
@@ -311,13 +312,13 @@ class StorageApplication(ServerNode, neo.storage.app.Application):
         checksum_dict = dict(dm.query("SELECT id, hash FROM data"))
         assert set(dm._uncommitted_data).issubset(checksum_dict)
         get = dm._uncommitted_data.get
-        return dict((str(v), get(k, 0)) for k, v in checksum_dict.iteritems())
+        return {str(v): get(k, 0) for k, v in checksum_dict.iteritems()}
 
 class ClientApplication(Node, neo.client.app.Application):
 
     @SerializedEventManager.decorate
-    def __init__(self, master_nodes, name):
-        super(ClientApplication, self).__init__(master_nodes, name)
+    def __init__(self, master_nodes, name, **kw):
+        super(ClientApplication, self).__init__(master_nodes, name, **kw)
         self.em._lock = threading.Lock()
 
     def setPoll(self, master=False):
@@ -545,7 +546,8 @@ class NEOCluster(object):
     def __init__(self, master_count=1, partitions=1, replicas=0, upstream=None,
                        adapter=os.getenv('NEO_TESTS_ADAPTER', 'SQLite'),
                        storage_count=None, db_list=None, clear_databases=True,
-                       db_user=DB_USER, db_password=''):
+                       db_user=DB_USER, db_password='', compress=True,
+                       importer=None):
         self.name = 'neo_%s' % self._allocate('name',
             lambda: random.randint(0, 100))
         master_list = [MasterApplication.newAddress()
@@ -573,11 +575,24 @@ class NEOCluster(object):
             db = os.path.join(getTempDirectory(), '%s.sqlite')
         else:
             assert False, adapter
+        if importer:
+            cfg = SafeConfigParser()
+            cfg.add_section("neo")
+            cfg.set("neo", "adapter", adapter)
+            cfg.set("neo", "database", db % tuple(db_list))
+            for name, zodb in importer:
+                cfg.add_section(name)
+                for x in zodb.iteritems():
+                    cfg.set(name, *x)
+            db = os.path.join(getTempDirectory(), '%s.conf')
+            with open(db % tuple(db_list), "w") as f:
+                cfg.write(f)
+            kw["getAdapter"] = "Importer"
         self.storage_list = [StorageApplication(getDatabase=db % x, **kw)
                              for x in db_list]
         self.admin_list = [AdminApplication(**kw)]
         self.client = ClientApplication(name=self.name,
-            master_nodes=self.master_nodes)
+            master_nodes=self.master_nodes, compress=compress)
         self.neoctl = NeoCTL(self.admin.getVirtualAddress())
 
     # A few shortcuts that work when there's only 1 master/storage/admin
